<?php
/*
 *  Copyright (C) 2006 CSIRO Australia
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  - Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *  
 *  - Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  
 *  - Neither the name of the CSIRO nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *  PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE ORGANISATION OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * \file Annodex.php
 * \author Chris Ford
 * \date January 2006
 */

require_once("phpannodex/Exceptions/AnxFileNotFoundException.php");
require_once("phpannodex/Exceptions/AnxException.php");

/**
 * \class Annodex
 * A class for encapsulating an Annodex handle. This class is abstract, so you
 * would only ever directly use subclasses of Annodex.
 */
abstract class Annodex {

  /** The MIME type of Annodex media.*/
  const MIME_TYPE = "application/x-annodex";

  /** Indicates to the constructor the Annodex is for reading. */ 
  const READ = ANX_READ;

  /** Indicates to the constructor the Annodex is for writing. */ 
  const WRITE = ANX_WRITE;

  /** A handle for the actual Annodex. */
  private $anx_handle;

  /** 
   * The constructor, for use by inheriting classes.
   * 
   * \param $type indicates whether the Annodex handle is a reader or a writer. 
   * The handle will default to a writer.
   *
   * \param $filename specifies an Annodex file that the Annodex will 
   * if no filename is given, the Annodex can still be used to output Annodex
   * media via AnnodexWriter::write_output().
   *
   * Throws AnxFileNotFoundException if a filename is specified and it cannot
   * be opened.
   */ 
  public function __construct( $type = ANX_WRITE, $filename = NULL ) {

    if( $type != self::READ && $type != self::WRITE ) {
      throw new AnxException("Unknown Annodex type.");
    }

    if( $filename ) {
      
      if( $this->anx_handle = anx_open( $filename, $type ) ) {
        return;
      } else {
        throw new AnxFileNotFoundException("Couldn't "
        ."open specified file.");
      }
    } else {
      $this->anx_handle = anx_new( $type );
    }

  }

  /**
   * An unremarkable getter.
   * \return A resource containing the actual Annodex handle.
   *
   */ 
  protected function get_anx_handle() {
    return $this->anx_handle;
  }

  /**
   * Tests to see whether the Annodex handle encapsulated by the Annodex is 
   * NULL. 
   *
   * \return True if the handle is NULL, false otherwise.
   */
  public function is_handle_null() {
    
    if( $this->get_anx_handle() == NULL ) {
      return TRUE;
    } else {
      return FALSE;
    }
  }

  /**
   * Checks the actual Annodex handle, and if it is in a state of error, throws
   * an appropriate AnxException.
   *
   * \return True if no AnxException is thrown.
   */
  protected function check_anx_error() {
    if( $this->last_error() != ANX_ERR_OK ) {
      throw new AnxException( $this->strerror() );
    }
    return true;
  } 

  /**
   * Parse a formatted time string.
   * 
   * \param $str A formatted time string.
   *
   * \return A double representation of the time string.
   */
  public static function parse_time( $str ) {
    return anx_parse_time( $str );
  }

  /**
   * Checks the Annodex handle for errors.
   *
   * \return The error code of the most recent error.
   */
  public function last_error() {
    return anx_last_error( $this->get_anx_handle() );
  }

  /**
   * Checks the Annodex handle for errors.
   *
   * \return A human readable error message.
   */
  public function strerror() {
    return anx_strerror( $this->get_anx_handle() );
  }

  /**
   * Flushes any as-yes unwritten data.
   *
   * \return True on success, false on failure.
   */
  public function flush() {
    return anx_flush( $this->get_anx_handle() );
  }

  /**
   * Closes the Annodex handle.
   *
   * \return True on success, false on failure.
   */
  public function close() {
    anx_close( $this->get_anx_handle() );
    return;
  }

  /**
   * Checks if the Annodex is ready.
   *
   * \return True if the Annodex is ready, false if it isn't.
   */
  public function is_ready() {
    return anx_ready( $this->get_anx_handle() );
  }

  /**
   * Queries if an annodex has reached its 'End Of Stream' markers.
   *
   * \return True if 'End Of Stream' has been encountered, false otherwise.
   */
  public function eos_encountered() {
    return anx_eos( $this->get_anx_handle() );
  }

  /**
   * Gets the presentation time of the Annodex.
   *
   * \return The presentation time as a double.
   */
  public function get_presentation_time() {
    return anx_get_presentation_time( $this->get_anx_handle() );
  }
  
  /**
   * Gets the basetime of the Annodex.
   *
   * \return The basetime as a double.
   */
  public function get_basetime() {
    return anx_get_basetime( $this->get_anx_handle() );
  }

  /**
   * Gets the bitrate of the Annodex.
   *
   * \return The bitrate as a double. A bitrate of -1 indicates that there is
   * an error.
   */
  public function get_bitrate() {
    return anx_get_bitrate( $this->get_anx_handle() );
  }
  
  /**
   * Gets the duration of the Annodex.
   *
   * \return The duration as a double.
   */
  public function get_duration() {
    return anx_get_duration( $this->get_anx_handle() );
  }

  /**
   * Gets the tell time of the Annodex.
   *
   * \return The tell time as a double.
   */
  public function get_tell_time() {
    return anx_tell_time( $this->get_anx_handle() );
  }

  /**
   * Gets the seek time of the Annodex.
   *
   * \return The seek time as a double.
   */
  public function get_seek_time() {
    return anx_seek_time( $this->get_anx_handle() );
  }

}

