<?php
/*
 *  Copyright (C) 2006 CSIRO Australia
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  - Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *  
 *  - Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  
 *  - Neither the name of the CSIRO nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *  PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE ORGANISATION OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

 /**
 * \file AnxInterval.php
 * \author Chris Ford
 * \date January 2006
 */

require_once("AnxSMPTE.php");
require_once("AnxNPT.php");
require_once("AnxUTC.php");
 
/**
 * \class AnxInterval
 * A class for encapsulating a time interval specified in a temporal URI. This
 * class is abstract, because it only makes sense to directly use specific time
 * schemes.
 */
class AnxInterval {

  /** The start of a media file. */
  const START = 0;

  /** The end of a media file. */
  const END = -1;

  /** Matches to any character, as per regular expressions. Used internally when
   *  parsing time strings.
   */
  const WILD_CARD = '?';

  /** Matches to any decimal number. Used internally when parsing time strings.
   */
  const NUMBER = 'N';

  /** The start of the interval encapsulated by this object. */
  private $start = self::START;
  
  /** The end of the interval encapsulated by this object. */
  private $end = self::END;

  /** 
   * The constructor, for use by inheriting classes.
   *
   * \param $string a string formatted according to some supported time scheme,
   * e.g. npt, smpte or utc.
   */
  public function __construct( $string ) {
  }

  /**
   * A factory method for producing subclasses of AnxInterval.
   *
   * \param $interval_string a string specified via a supported time scheme.
   * 
   * \return An object belonging to a subclass of AnxInterval that encapsulates
   * the interval specified by the argument $interval_string.
   */
  public static function factory( $interval_string ) {

    /* If nothing is specified, stick with the defaults. */
    if( !$interval_string ) {
      return new AnxInterval( $interval_string );
    }

  /* Should new kinds of time scheme become supported, they need to be
   * added here.
   */
    if( self::is_prefix( "smpte", $interval_string ) ) {
      return new AnxSMPTE( $interval_string );
    } else if( self::is_prefix( "utc", $interval_string)) {
      return new AnxUTC( $interval_string );
    } else if( self::is_prefix( "npt", $interval_string)) {
      return new AnxNPT( $interval_string );
    } else {
      return new AnxNPT( AnxNPT::PREFIX.$interval_string );
    }
  }

  /**
   * A standard getter method.
   * \return A double corresponding to the start of the interval.
   */ 
  public function get_start() {
    return (double) $this->start;
  }

  /**
   * A standard getter method.
   * \return a double corresponding to the end of the interval (in seconds).
   */ 
  public function get_end() {
    return (double) $this->end;
  }

  /**
   * A standard setter method.
   * \param $start a double that will become the start of the interval (in
   * seconds).
   */ 
  public function set_start( $start ) {
    $this->start = $start;
  }

  /**
   * A standard setter method.
   *
   * \param $end a double that will become the end of the interval.
   */ 
  public function set_end( $end ) {
    $this->end = $end;
  }

  /** 
   * A method for parsing time strings.
   * 
   * \param $string is the raw time string to be parsed.
   * \param $template is a template for a particular time scheme, 
   * e.g. "smpte-drop-N:N:N:N:N-N:N:N:N".
   *
   * \return An array containing the wildcarded elements in the time string in
   * the order in which they appear in the string.
   * e.g. passing "smpte-drop-20:0:0:3:4-0:1:0:2" will return an array with the
   * elements 20, 0, 0, 3, 0, 1, 0, 2 in that order.
   */
  protected static function match( $string, $template ) {

    $string_array = str_split( $string );
    $template_array = str_split( $template );

    while( $t_i < strlen($template) && $str_i < strlen($string) ) {
      switch( $template[$t_i] ) {
        case self::WILD_CARD:
        if( $in_word ) {
          $tokens[$count] = 
            $tokens[$count].$string[$str_i];
          $count = count($tokens);
        } else {
          $tokens[] = $string[$str_i];
        }
        $str_i++;
        $t_i++;
        $in_word = true;
        break;

        case self::NUMBER:
        $str_i_start = $str_i;
        while( $str_i<strlen($string)
          && (ctype_digit($string[$str_i]) 
            || $string[$str_i] == '.') ) {
          $str_i++; 
        }
        $number = substr($string, $str_i_start, 
              $str_i-$str_i_start);
        if( !is_numeric( $number ) ) {
          return false;
        }
        $tokens[] = $number;
        $in_word = false;
        $t_i++;
        break;

        case $string[$str_i] != $template[$t_i]:
             return false;
             break;

        default:
        $t_i++;
        $str_i++;
        break;
      }        
    }
    /* Make sure everything was consumed. */
    if( $str_i != strlen( $string ) || $t_i != strlen($template) ) {
      return false;
    } else {
      return $tokens;
    }
      
  }

  /**
   * A utility method that checks if a string is a prefix of another string. 
   * Used internally to check what kind of interval is specified in a raw
   * string.
   *
   * \param $prefix is a string prefix.
   * \param $string is the string that may or may not have $prefix at the front.
   *
   * \return True if $prefix is a prefix of $string, false otherwise.
   */
  protected static function is_prefix( $prefix, $string ) {
    if( $prefix == substr( $string, 0, strlen( $prefix ) ) ) {
           return true;
    } else {
      return false;
    } 
  } 

}
