<?php
/*
 *  Copyright (C) 2006 CSIRO Australia
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  - Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *  
 *  - Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  
 *  - Neither the name of the CSIRO nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *  PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE ORGANISATION OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * \file AnxSMPTE.php
 * \author Chris Ford
 * \date January 2006
 */

require_once("phpannodex/Exceptions/AnxBadFormatException.php");

/**
 * \class AnxSMPTE
 * A class for encapsulating intervals specified with the smpte time scheme.
 */
class AnxSMPTE extends AnxInterval {

  /** The prefix that distinguishes smpte intervals. */
  const PREFIX = "smpte";

  /**
   * The constructor.
   *
   * \param $string a string specifying a smpte interval.
   *
   * \return TRUE on success, and throws an AnxBadFormatException if $string is
   * not properly formatted.
   */
  public function __construct( $string ) {
    parent::__construct( $string );

    $n = parent::NUMBER;

    /* Strings matching the various kinds of smpte strings. */
    $drop_open = self::PREFIX."-$n-drop:$n:$n:$n:$n";
    $drop_closed = self::PREFIX."-$n-drop:$n:$n:$n:$n-$n:$n:$n:$n";
    $plain_open = self::PREFIX."-$n:$n:$n:$n:$n";
    $plain_closed = self::PREFIX."-$n:$n:$n:$n:$n-$n:$n:$n:$n";

    /* Match the bits of the time and convert to seconds. */  
    if( $tokens = parent::match( $string, $plain_open ) ) {
      if( ($fps = $tokens[0]) <= 0 ) {
        self::fps_invalid( $string );
      }
      $start = 60*60*$tokens[1] + 60*$tokens[2] + $tokens[3]
          + $tokens[4]/$fps;
      $end = parent::DEFAULT_END;
    } else if( $tokens = parent::match( $string, $plain_closed ) ) {
      if( ($fps = $tokens[0]) <= 0 ) {
        self::fps_invalid( $string );
      }
      $start = 60*60*$tokens[1] + 60*$tokens[2] + $tokens[3]
          + $tokens[4]/$fps;
      $end = 60*60*$tokens[5] + 60*$tokens[6] + $tokens[7]
          + $tokens[8]/$fps;
    } else if( $tokens = parent::match( $string, $drop_open ) ) {
      if( ($fps = $tokens[0]) <= 0 ) {
        self::fps_invalid( $string );
      }
      $start = 60*60*$tokens[1] + 60*$tokens[2] + $tokens[3]
          + $tokens[4]/$fps;
      $end = parent::DEFAULT_END;
    } else if( $tokens = parent::match( $string, $drop_closed ) ) {
      if( ($fps = $tokens[0]) <= 0 ) {
        self::fps_invalid( $string );
      }
      $start = 60*60*$tokens[1] + 60*$tokens[2] + $tokens[3]
          + $tokens[4]/$fps;
      $end = 60*60*$tokens[5] + 60*$tokens[6] + $tokens[7]
          + $tokens[8]/$fps;
    } else {
      throw new AnxBadFormatException("Bad smpte time: "
            ."'$string'");
    }

    $this->set_start($start);
    $this->set_end($end);
    
    return; 

  }

  /**
   * A utility method for conveniently throwing an AnxBadFormatException when
   * frames per second is invalid.
   *
   * \param $string the original time string.
   *
   * \return always throws an AnxBadFormatException.
   */
  protected static function fps_invalid( $string ) {
    throw new AnxBadFormatException(
       "Frames per second must be positive for "
       ."an smpte time. '$string' is not valid.");
  }
}
